//SVGA.H - Rotinas para manipulacao do Video SVGA

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>


//Estrutura de Informacoes VESA
struct	vesainfo {
   char	assinatura[4];
   int  versao;
   char *infofab;
   char	capacidade[4];
   int  *modo;
   char	reservado[246];
};

//Estrutura de Informacoes dos Modos VESA
struct	vesa_modo_info {
	int	attributo;
	char	janelaA;
	char	janelaB;
	int	granulacao;
	int	tamanho;
	int	segmento_a;
	int	segmento_b;
	void    (*seta_janela)();
	int	bytes_linha;
	int	xpixels;
	int	ypixels;
	char	altura;
	char	largura;
	char	planos;
	char	bits;
	char	bancos;
	char	memoria;
	char	tam_banco;
	char	reservado[246];
};

struct vesainfo       vesa_info;
struct vesa_modo_info vmodo_info;


//Funcoes Para Manipulacao dos Modos Vesa

// Le_Info_Vesa
// Objetivo  : Ler as Informacoes do modo de Video Desejado
// Parametros:
//     Modo : Modo de video desejado
// Retorno   : Ponteiro para uma estrutura Vesa_Modo_Info
struct vesa_modo_info *le_info_vesa(int modo)
{
    union REGS r;
    struct SREGS s;

    r.h.ah=0x4f;
    r.h.al=0x01;
    r.x.cx=modo;
    s.es=FP_SEG(&vmodo_info);
    r.x.di=FP_OFF(&vmodo_info);
    int86x(0x10,&r,&r,&s);
    if ( r.x.ax == 0x004F )
       return(&vmodo_info);
    else
       return(NULL);
}


// Testa_Modo_Vesa
// Objetivo  : Verificar se deteminado modo esta disponivel
// Parametros:
//     Modo : Modo de video desejado
// Retorno   : 1 em caso de modo disponivel, 0 em caso de erro
int testa_modo_vesa(int modo)
{
  union REGS r;
  struct SREGS s;

  r.h.ah=0x4f;
  r.h.al=0;
  s.es=FP_SEG(&vesa_info);
  r.x.di=FP_OFF(&vesa_info);
  int86x(0x10,&r,&r,&s);
  if (r.x.ax != 0x004f)
     return(0);
  if (strncmp(vesa_info.assinatura,"VESA",4) == 0) {
     while ((*(vesa_info.modo) != EOF) && (*(vesa_info.modo) != modo) )
	vesa_info.modo++;
     if (*vesa_info.modo == modo)
	return(1);
     else
	return(0);
  }
  else
     return(0);

}


// Seta_Modo_Vesa
// Objetivo  : Setar o modo de video
// Parametros:
//     Modo : Modo de video desejado
// Retorno   : 1, em caso de sucesso ou 0 em caso de erro
int seta_modo_vesa(int modo)
{
    union REGS r;

    r.x.ax=0x4f02;
    r.x.bx=modo;
    int86(0x10,&r,&r);
    if ( r.h.ah == 0 )
       return(1);
    else
       return(0);
}


// Testa_Vesa
// Objetivo  : Verificar se um driver Vesa esta Instalado
// Retorno   : 1 em caso de sucesso, 0 em caso de erro
int testa_vesa(void)
{
    union   REGS    r;
    struct  SREGS   s;

    r.x.ax = 0x4F00;
    s.es = FP_SEG(&vesa_info);
    r.x.di= FP_OFF(&vesa_info);
    int86x(0x10, &r, &r, &s);

    if ((r.x.ax != 0x004F) ||
	(strncmp(vesa_info.assinatura,"VESA",4) != 0) ) {
	return(0);
    }
    return(1);
}



// Muda_Banco
// Objetivo  : Calcula o endereco Desejado e muda o Banco SVGA
// Parametros:
//     X, Y: Coordenadas horizontal e Vertical
//     Colunas: Numero de Colunas do Modo Desejado
// Retorno   : Ponteiro para uma estrutura Vesa_Modo_Info
int muda_banco(int x, int y, int colunas)
{

  union REGS r;
  long endereco;
  int banco=0;

  endereco = (long) y*colunas+x;       //Calcula Endereco Absoluto
  banco = endereco / 65536L;           //Calcula Banco
  endereco = endereco - banco*65536L;  //Calcula Deslocamento

  r.x.dx = banco;
  r.x.bx = 0;
  r.x.ax = 0x4f05;    //Chama servico 4fh, subservico 5
  int86(0x10,&r,&r);  //interrupcao 10h

  return endereco;
}